@@ -10,7 +10,7 @@
 /**
  * Names of field flags.
  */
-$pma_mysqli_flag_names = array(
+$GLOBALS['pma_mysqli_flag_names'] = array(
     MYSQLI_NUM_FLAG => 'num',
     MYSQLI_PART_KEY_FLAG => 'part_key',
     MYSQLI_SET_FLAG => 'set',
@@ -24,4 +24,4 @@ $pma_mysqli_flag_names = array(
     MYSQLI_UNIQUE_KEY_FLAG => 'unique_key',
     MYSQLI_PRI_KEY_FLAG => 'primary_key',
     MYSQLI_NOT_NULL_FLAG => 'not_null',
-);
\ No newline at end of file
+);
