@@ -8,7 +8,7 @@
 use PMA\libraries\Message;
 use PMA\libraries\Table;
 use PMA\libraries\RecentFavoriteTable;
-use SqlParser\Statements\CreateStatement;
+use PMA\libraries\URL;
 
 /**
  * Executes a query as controluser if possible, otherwise as normal user
@@ -191,7 +191,7 @@ function PMA_getRelationsParamDiagnostic($cfgRelation)
             $retval .= '<tr><td colspan=2 class="left error">';
             $retval .=  __(
                 'Please see the documentation on how to'
-                . ' update your column_info table. '
+                . ' update your column_info table.'
             );
             $retval .= PMA\libraries\Util::showDocu(
                 'config',
@@ -756,29 +756,28 @@ function PMA_getForeigners($db, $table, $column = '', $source = 'both')
                 `foreign_field`
             FROM ' . PMA\libraries\Util::backquote($cfgRelation['db'])
                 . '.' . PMA\libraries\Util::backquote($cfgRelation['relation']) . '
-            WHERE `master_db`    = \'' . PMA\libraries\Util::sqlAddSlashes($db) . '\'
-                AND `master_table` = \'' . PMA\libraries\Util::sqlAddSlashes($table)
+            WHERE `master_db`    = \'' . $GLOBALS['dbi']->escapeString($db) . '\'
+                AND `master_table` = \'' . $GLOBALS['dbi']->escapeString($table)
             . '\' ';
-        if (mb_strlen($column)) {
+        if (strlen($column) > 0) {
             $rel_query .= ' AND `master_field` = '
-                . '\'' . PMA\libraries\Util::sqlAddSlashes($column) . '\'';
+                . '\'' . $GLOBALS['dbi']->escapeString($column) . '\'';
         }
         $foreign = $GLOBALS['dbi']->fetchResult(
             $rel_query, 'master_field', null, $GLOBALS['controllink']
         );
     }
 
-    if (($source == 'both' || $source == 'foreign') && mb_strlen($table)
-    ) {
+    if (($source == 'both' || $source == 'foreign') && strlen($table) > 0) {
         $tableObj = new Table($table, $db);
         $show_create_table = $tableObj->showCreate();
         if ($show_create_table) {
-            $parser = new SqlParser\Parser($show_create_table);
+            $parser = new \PhpMyAdmin\SqlParser\Parser($show_create_table);
             /**
-             * @var CreateStatement $stmt
+             * @var \PhpMyAdmin\SqlParser\Statements\CreateStatement $stmt
              */
             $stmt = $parser->statements[0];
-            $foreign['foreign_keys_data'] = SqlParser\Utils\Table::getForeignKeys(
+            $foreign['foreign_keys_data'] = \PhpMyAdmin\SqlParser\Utils\Table::getForeignKeys(
                 $stmt
             );
         }
@@ -801,9 +800,9 @@ function PMA_getForeigners($db, $table, $column = '', $source = 'both')
         }
         if (isset($GLOBALS[$relations_key][$table])) {
             foreach ($GLOBALS[$relations_key][$table] as $field => $relations) {
-                if ((! mb_strlen($column) || $column == $field)
+                if ((strlen($column) === 0 || $column == $field)
                     && (! isset($foreign[$field])
-                    || ! mb_strlen($foreign[$field]))
+                    || strlen($foreign[$field]) === 0)
                 ) {
                     $foreign[$field] = $relations;
                 }
@@ -836,8 +835,8 @@ function PMA_getDisplayField($db, $table)
             SELECT `display_field`
             FROM ' . PMA\libraries\Util::backquote($cfgRelation['db'])
                 . '.' . PMA\libraries\Util::backquote($cfgRelation['table_info']) . '
-            WHERE `db_name`    = \'' . PMA\libraries\Util::sqlAddSlashes($db) . '\'
-                AND `table_name` = \'' . PMA\libraries\Util::sqlAddSlashes($table)
+            WHERE `db_name`    = \'' . $GLOBALS['dbi']->escapeString($db) . '\'
+                AND `table_name` = \'' . $GLOBALS['dbi']->escapeString($table)
             . '\'';
 
         $row = $GLOBALS['dbi']->fetchSingleRow(
@@ -860,6 +859,18 @@ function PMA_getDisplayField($db, $table)
         }
     }
 
+    /**
+     * Pick first char field
+     */
+    $columns = $GLOBALS['dbi']->getColumnsFull($db, $table);
+    if ($columns) {
+        foreach ($columns as $column) {
+            if ($GLOBALS['PMA_Types']->getTypeClass($column['DATA_TYPE']) == 'CHAR') {
+                return $column['COLUMN_NAME'];
+            }
+        }
+    }
+
     /**
      * No Luck...
      */
@@ -919,7 +930,7 @@ function PMA_getDbComment($db)
             FROM " . PMA\libraries\Util::backquote($cfgRelation['db'])
                 . "." . PMA\libraries\Util::backquote($cfgRelation['column_info'])
                 . "
-            WHERE db_name     = '" . PMA\libraries\Util::sqlAddSlashes($db) . "'
+            WHERE db_name     = '" . $GLOBALS['dbi']->escapeString($db) . "'
                 AND table_name  = ''
                 AND column_name = '(db_comment)'";
         $com_rs = PMA_queryAsControlUser(
@@ -989,23 +1000,23 @@ function PMA_setDbComment($db, $comment = '')
         return false;
     }
 
-    if (mb_strlen($comment)) {
+    if (strlen($comment) > 0) {
         $upd_query = 'INSERT INTO '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['column_info'])
             . ' (`db_name`, `table_name`, `column_name`, `comment`)'
             . ' VALUES (\''
-            . PMA\libraries\Util::sqlAddSlashes($db)
+            . $GLOBALS['dbi']->escapeString($db)
             . "', '', '(db_comment)', '"
-            . PMA\libraries\Util::sqlAddSlashes($comment)
+            . $GLOBALS['dbi']->escapeString($comment)
             . "') "
             . ' ON DUPLICATE KEY UPDATE '
-            . "`comment` = '" . PMA\libraries\Util::sqlAddSlashes($comment) . "'";
+            . "`comment` = '" . $GLOBALS['dbi']->escapeString($comment) . "'";
     } else {
         $upd_query = 'DELETE FROM '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['column_info'])
-            . ' WHERE `db_name`     = \'' . PMA\libraries\Util::sqlAddSlashes($db)
+            . ' WHERE `db_name`     = \'' . $GLOBALS['dbi']->escapeString($db)
             . '\'
                 AND `table_name`  = \'\'
                 AND `column_name` = \'(db_comment)\'';
@@ -1071,11 +1082,11 @@ function PMA_setHistory($db, $table, $username, $sqlquery)
                 `timevalue`,
                 `sqlquery`)
         VALUES
-              (\'' . PMA\libraries\Util::sqlAddSlashes($username) . '\',
-               \'' . PMA\libraries\Util::sqlAddSlashes($db) . '\',
-               \'' . PMA\libraries\Util::sqlAddSlashes($table) . '\',
+              (\'' . $GLOBALS['dbi']->escapeString($username) . '\',
+               \'' . $GLOBALS['dbi']->escapeString($db) . '\',
+               \'' . $GLOBALS['dbi']->escapeString($table) . '\',
                NOW(),
-               \'' . PMA\libraries\Util::sqlAddSlashes($sqlquery) . '\')'
+               \'' . $GLOBALS['dbi']->escapeString($sqlquery) . '\')'
     );
 
     PMA_purgeHistory($username);
@@ -1117,7 +1128,7 @@ function PMA_getHistory($username)
                 `timevalue`
            FROM ' . PMA\libraries\Util::backquote($cfgRelation['db'])
             . '.' . PMA\libraries\Util::backquote($cfgRelation['history']) . '
-          WHERE `username` = \'' . PMA\libraries\Util::sqlAddSlashes($username) . '\'
+          WHERE `username` = \'' . $GLOBALS['dbi']->escapeString($username) . '\'
        ORDER BY `id` DESC';
 
     return $GLOBALS['dbi']->fetchResult(
@@ -1152,7 +1163,7 @@ function PMA_purgeHistory($username)
         SELECT `timevalue`
         FROM ' . PMA\libraries\Util::backquote($cfgRelation['db'])
             . '.' . PMA\libraries\Util::backquote($cfgRelation['history']) . '
-        WHERE `username` = \'' . PMA\libraries\Util::sqlAddSlashes($username) . '\'
+        WHERE `username` = \'' . $GLOBALS['dbi']->escapeString($username) . '\'
         ORDER BY `timevalue` DESC
         LIMIT ' . $GLOBALS['cfg']['QueryHistoryMax'] . ', 1';
 
@@ -1163,7 +1174,7 @@ function PMA_purgeHistory($username)
             'DELETE FROM '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['history']) . '
-              WHERE `username` = \'' . PMA\libraries\Util::sqlAddSlashes($username)
+              WHERE `username` = \'' . $GLOBALS['dbi']->escapeString($username)
             . '\'
                 AND `timevalue` <= \'' . $max_time . '\''
         );
@@ -1330,13 +1341,17 @@ function PMA_foreignDropdown($disp_row, $foreign_field, $foreign_display, $data,
  * @param bool          $override_total whether to override the total
  * @param string        $foreign_filter a possible filter
  * @param string        $foreign_limit  a possible LIMIT clause
+ * @param bool          $get_total      optional, whether to get total num of rows
+ *                                      in $foreignData['the_total;]
+ *                                      (has an effect of performance)
  *
  * @return array    data about the foreign keys
  *
  * @access  public
  */
 function PMA_getForeignData(
-    $foreigners, $field, $override_total, $foreign_filter, $foreign_limit
+    $foreigners, $field, $override_total,
+    $foreign_filter, $foreign_limit, $get_total=false
 ) {
     // we always show the foreign field in the drop-down; if a display
     // field is defined, we show it besides the foreign field
@@ -1362,11 +1377,13 @@ function PMA_getForeignData(
         // We could also do the SELECT anyway, with a LIMIT, and ensure that
         // the current value of the field is one of the choices.
 
-        $the_total = $GLOBALS['dbi']->getTable($foreign_db, $foreign_table)
-            ->countRecords(true);
+        // Check if table has more rows than specified by
+        // $GLOBALS['cfg']['ForeignKeyMaxLimit']
+        $moreThanLimit = $GLOBALS['dbi']->getTable($foreign_db, $foreign_table)
+            ->checkIfMinRecordsExist($GLOBALS['cfg']['ForeignKeyMaxLimit']);
 
         if ($override_total == true
-            || $the_total < $GLOBALS['cfg']['ForeignKeyMaxLimit']
+            || !$moreThanLimit
         ) {
             // foreign_display can be false if no display field defined:
             $foreign_display = PMA_getDisplayField($foreign_db, $foreign_table);
@@ -1381,24 +1398,19 @@ function PMA_getForeignData(
                 . '.' . PMA\libraries\Util::backquote($foreign_table);
             $f_query_filter = empty($foreign_filter) ? '' : ' WHERE '
                 . PMA\libraries\Util::backquote($foreign_field)
-                . ' LIKE "%' . PMA\libraries\Util::sqlAddSlashes(
-                    $foreign_filter,
-                    true
-                ) . '%"'
+                . ' LIKE "%' . $GLOBALS['dbi']->escapeString($foreign_filter) . '%"'
                 . (
                 ($foreign_display == false)
                     ? ''
                     : ' OR ' . PMA\libraries\Util::backquote($foreign_display)
-                    . ' LIKE "%' . PMA\libraries\Util::sqlAddSlashes(
-                        $foreign_filter,
-                        true
-                    )
+                    . ' LIKE "%' . $GLOBALS['dbi']->escapeString($foreign_filter)
                     . '%"'
                 );
             $f_query_order = ($foreign_display == false) ? '' :' ORDER BY '
                 . PMA\libraries\Util::backquote($foreign_table) . '.'
                 . PMA\libraries\Util::backquote($foreign_display);
-            $f_query_limit = isset($foreign_limit) ? $foreign_limit : '';
+
+            $f_query_limit = ! empty($foreign_limit) ? ($foreign_limit) : '';
 
             if (!empty($foreign_filter)) {
                 $the_total = $GLOBALS['dbi']->fetchValue(
@@ -1436,6 +1448,11 @@ function PMA_getForeignData(
         }
     } while (false);
 
+    if ($get_total) {
+        $the_total = $GLOBALS['dbi']->getTable($foreign_db, $foreign_table)
+            ->countRecords(true);
+    }
+
     $foreignData = array();
     $foreignData['foreign_link'] = $foreign_link;
     $foreignData['the_total'] = isset($the_total) ? $the_total : null;
@@ -1444,6 +1461,7 @@ function PMA_getForeignData(
     );
     $foreignData['disp_row'] = isset($disp_row) ? $disp_row : null;
     $foreignData['foreign_field'] = isset($foreign_field) ? $foreign_field : null;
+
     return $foreignData;
 } // end of 'PMA_getForeignData()' function
 
@@ -1467,14 +1485,14 @@ function PMA_REL_renameField($db, $table, $field, $new_name)
         $table_query = 'UPDATE '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['table_info'])
-            . '   SET display_field = \'' . PMA\libraries\Util::sqlAddSlashes(
+            . '   SET display_field = \'' . $GLOBALS['dbi']->escapeString(
                 $new_name
             ) . '\''
-            . ' WHERE db_name       = \'' . PMA\libraries\Util::sqlAddSlashes($db)
+            . ' WHERE db_name       = \'' . $GLOBALS['dbi']->escapeString($db)
             . '\''
-            . '   AND table_name    = \'' . PMA\libraries\Util::sqlAddSlashes($table)
+            . '   AND table_name    = \'' . $GLOBALS['dbi']->escapeString($table)
             . '\''
-            . '   AND display_field = \'' . PMA\libraries\Util::sqlAddSlashes($field)
+            . '   AND display_field = \'' . $GLOBALS['dbi']->escapeString($field)
             . '\'';
         PMA_queryAsControlUser($table_query);
     }
@@ -1483,28 +1501,28 @@ function PMA_REL_renameField($db, $table, $field, $new_name)
         $table_query = 'UPDATE '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['relation'])
-            . '   SET master_field = \'' . PMA\libraries\Util::sqlAddSlashes(
+            . '   SET master_field = \'' . $GLOBALS['dbi']->escapeString(
                 $new_name
             ) . '\''
-            . ' WHERE master_db    = \'' . PMA\libraries\Util::sqlAddSlashes($db)
+            . ' WHERE master_db    = \'' . $GLOBALS['dbi']->escapeString($db)
             . '\''
-            . '   AND master_table = \'' . PMA\libraries\Util::sqlAddSlashes($table)
+            . '   AND master_table = \'' . $GLOBALS['dbi']->escapeString($table)
             . '\''
-            . '   AND master_field = \'' . PMA\libraries\Util::sqlAddSlashes($field)
+            . '   AND master_field = \'' . $GLOBALS['dbi']->escapeString($field)
             . '\'';
         PMA_queryAsControlUser($table_query);
 
         $table_query = 'UPDATE '
             . PMA\libraries\Util::backquote($cfgRelation['db']) . '.'
             . PMA\libraries\Util::backquote($cfgRelation['relation'])
-            . '   SET foreign_field = \'' . PMA\libraries\Util::sqlAddSlashes(
+            . '   SET foreign_field = \'' . $GLOBALS['dbi']->escapeString(
                 $new_name
             ) . '\''
-            . ' WHERE foreign_db    = \'' . PMA\libraries\Util::sqlAddSlashes($db)
+            . ' WHERE foreign_db    = \'' . $GLOBALS['dbi']->escapeString($db)
             . '\''
-            . '   AND foreign_table = \'' . PMA\libraries\Util::sqlAddSlashes($table)
+            . '   AND foreign_table = \'' . $GLOBALS['dbi']->escapeString($table)
             . '\''
-            . '   AND foreign_field = \'' . PMA\libraries\Util::sqlAddSlashes($field)
+            . '   AND foreign_field = \'' . $GLOBALS['dbi']->escapeString($field)
             . '\'';
         PMA_queryAsControlUser($table_query);
 
@@ -1534,14 +1552,14 @@ function PMA_REL_renameSingleTable($table,
         . PMA\libraries\Util::backquote($GLOBALS['cfgRelation']['db']) . '.'
         . PMA\libraries\Util::backquote($GLOBALS['cfgRelation'][$table])
         . ' SET '
-        . $db_field . ' = \'' . PMA\libraries\Util::sqlAddSlashes($target_db)
+        . $db_field . ' = \'' . $GLOBALS['dbi']->escapeString($target_db)
         . '\', '
-        . $table_field . ' = \'' . PMA\libraries\Util::sqlAddSlashes($target_table)
+        . $table_field . ' = \'' . $GLOBALS['dbi']->escapeString($target_table)
         . '\''
         . ' WHERE '
-        . $db_field . '  = \'' . PMA\libraries\Util::sqlAddSlashes($source_db) . '\''
+        . $db_field . '  = \'' . $GLOBALS['dbi']->escapeString($source_db) . '\''
         . ' AND '
-        . $table_field . ' = \'' . PMA\libraries\Util::sqlAddSlashes($source_table)
+        . $table_field . ' = \'' . $GLOBALS['dbi']->escapeString($source_table)
         . '\'';
     PMA_queryAsControlUser($query);
 }
@@ -1614,8 +1632,8 @@ function PMA_REL_renameTable($source_db, $target_db, $source_table, $target_tabl
             $remove_query = "DELETE FROM "
                 . PMA\libraries\Util::backquote($GLOBALS['cfgRelation']['db']) . "."
                 . PMA\libraries\Util::backquote($GLOBALS['cfgRelation']['table_coords'])
-                . " WHERE db_name  = '" . PMA\libraries\Util::sqlAddSlashes($source_db) . "'"
-                . " AND table_name = '" . PMA\libraries\Util::sqlAddSlashes($source_table)
+                . " WHERE db_name  = '" . $GLOBALS['dbi']->escapeString($source_db) . "'"
+                . " AND table_name = '" . $GLOBALS['dbi']->escapeString($source_table)
                 . "'";
             PMA_queryAsControlUser($remove_query);
         }
@@ -1645,13 +1663,13 @@ function PMA_REL_renameTable($source_db, $target_db, $source_table, $target_tabl
             . PMA\libraries\Util::backquote(
                 $GLOBALS['cfgRelation']['navigationhiding']
             )
-            . " SET db_name = '" . PMA\libraries\Util::sqlAddSlashes($target_db)
+            . " SET db_name = '" . $GLOBALS['dbi']->escapeString($target_db)
             . "',"
-            . " item_name = '" . PMA\libraries\Util::sqlAddSlashes($target_table)
+            . " item_name = '" . $GLOBALS['dbi']->escapeString($target_table)
             . "'"
-            . " WHERE db_name  = '" . PMA\libraries\Util::sqlAddSlashes($source_db)
+            . " WHERE db_name  = '" . $GLOBALS['dbi']->escapeString($source_db)
             . "'"
-            . " AND item_name = '" . PMA\libraries\Util::sqlAddSlashes($source_table)
+            . " AND item_name = '" . $GLOBALS['dbi']->escapeString($source_table)
             . "'"
             . " AND item_type = 'table'";
         PMA_queryAsControlUser($query);
@@ -1665,7 +1683,7 @@ function PMA_REL_renameTable($source_db, $target_db, $source_table, $target_tabl
  * @param array  $cfgRelation Relation configuration
  * @param string $db          database name
  *
- * @return string   $pdf_page_number
+ * @return int $pdf_page_number
  */
 function PMA_REL_createPage($newpage, $cfgRelation, $db)
 {
@@ -1677,8 +1695,8 @@ function PMA_REL_createPage($newpage, $cfgRelation, $db)
         . PMA\libraries\Util::backquote($cfgRelation['pdf_pages'])
         . ' (db_name, page_descr)'
         . ' VALUES (\''
-        . PMA\libraries\Util::sqlAddSlashes($db) . '\', \''
-        . PMA\libraries\Util::sqlAddSlashes($newpage) . '\')';
+        . $GLOBALS['dbi']->escapeString($db) . '\', \''
+        . $GLOBALS['dbi']->escapeString($newpage) . '\')';
     PMA_queryAsControlUser($ins_query, false);
 
     return $GLOBALS['dbi']->insertId(
@@ -1704,12 +1722,12 @@ function PMA_getChildReferences($db, $table, $column = '')
             . " `table_schema`, `referenced_column_name`"
             . " FROM `information_schema`.`key_column_usage`"
             . " WHERE `referenced_table_name` = '"
-            . PMA\libraries\Util::sqlAddSlashes($table) . "'"
+            . $GLOBALS['dbi']->escapeString($table) . "'"
             . " AND `referenced_table_schema` = '"
-            . PMA\libraries\Util::sqlAddSlashes($db) . "'";
+            . $GLOBALS['dbi']->escapeString($db) . "'";
         if ($column) {
             $rel_query .= " AND `referenced_column_name` = '"
-                . PMA\libraries\Util::sqlAddSlashes($column) . "'";
+                . $GLOBALS['dbi']->escapeString($column) . "'";
         }
 
         $child_references = $GLOBALS['dbi']->fetchResult(
@@ -1972,7 +1990,7 @@ function PMA_getHtmlFixPMATables($allTables, $createDb = false)
 {
     $retval = '';
 
-    $url_query = PMA_URL_getCommon(array('db' => $GLOBALS['db']));
+    $url_query = URL::getCommon(array('db' => $GLOBALS['db']));
     if ($allTables) {
         if ($createDb) {
             $url_query .= '&amp;goto=db_operations.php&amp;create_pmadb=1';
@@ -1997,11 +2015,8 @@ function PMA_getHtmlFixPMATables($allTables, $createDb = false)
             __('%sCreate%s missing phpMyAdmin configuration storage tables.')
         );
     }
-    $message->addParam(
-        '<a href="./chk_rel.php' . $url_query . '">',
-        false
-    );
-    $message->addParam('</a>', false);
+    $message->addParamHtml('<a href="./chk_rel.php' . $url_query . '">');
+    $message->addParamHtml('</a>');
 
     $retval .= $message->getDisplay();
 
